import wx
import wx.lib.mixins.listctrl as listmix
import ssglob
from ui.ids import *

class BaseList(wx.ListCtrl, listmix.ColumnSorterMixin):
    # Semi-abstract class to be inherited by the two lists
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1, style=wx.LC_REPORT|wx.SUNKEN_BORDER|wx.LC_VIRTUAL|wx.LC_SINGLE_SEL|wx.VSCROLL)

        self.SetItemCount(self.GetItemCount())
        self.selected = None
        self.parent = parent

        self.CreateColumns()
        self.RegisterEvents()

        listmix.ColumnSorterMixin.__init__(self, self.GetColumnCount())
        # Maps sorted index to actual index in text or sound data lists.
        self.indexMap = []
        self.InitIndexMap()

    def CreateColumns(self):
        pass

    def RegisterEvents(self):
        pass

    def OnGetItemText(self, item, column):
        pass

    def GetItemCount(self):
        return 0

    def SelectItem(self, event):
        self.selected = event.GetIndex()

    def SelectItemManual(self, ind):
        self.selected = ind
        self.Select(self.selected, 1) # select

    def SelectNextItem(self):
        oldsel = self.selected
        if oldsel != None and oldsel < self.GetItemCount() - 1:
            self.Deselect()
            self.SelectItemManual(oldsel +1 )
            # scroll if we're getting too near the bottom of the view
            if oldsel + 1 >= (self.GetTopItem() + self.GetCountPerPage() - 2):
                self.ScrollLines(1)

    def CreateList(self):
        """ Regenerate a list when a new file is opened. Takes the filename of the new file."""
        self.DeleteAllItems()
        self.SetItemCount(self.GetItemCount())
        if self.selected != None:
            self.Deselect()
        self.InitIndexMap()
        self.SortItems()
        self.RefreshViewableItems()

    def Deselect(self):
        ##if self.selected != None:
            self.Select(self.selected, 0) # deselect
            self.selected = None

    def RefreshViewableItems(self):
        """ Updates items from the index of the topmost visible item to the index of the topmost visible item plus the number of items visible."""
        self.RefreshItems(self.GetTopItem(), self.GetTopItem() + self.GetCountPerPage()) #redraw

    def RefreshSelectedItem(self):
        print self.selected
        self.RefreshItem(self.selected)

    def observe(self, obj):
        #self.CreateList('meow')
        self.RefreshViewableItems()

    def InitIndexMap(self):
        pass

    def GetListCtrl(self):
        return self

    def SelectedToIndex(self):
        return self.indexMap[self.selected]

    def EnsureVisibleAtTop(self, item):
        self.EnsureVisible(item + self.GetCountPerPage() - 1)

class LeftList(BaseList):

    def GetItemCount(self):
        if ssglob.gTextData is None:
            return 0
        return len(ssglob.gTextData)

    def OnGetItemText(self, item, column):
        if ssglob.gTextData is None:
            return None
        item = self.indexMap[item]
        if column == 0:
            return str(item).zfill(4)
        elif column == 1:
            stringy = ssglob.gTextData[item]
            return stringy.rstrip('\n') # remove newline
        elif column == 2: # Indicate whether a sound file has been assigned to text
            if ssglob.gSpeechMap is None:
                return "--"
            sound_list = ssglob.gSpeechMap.get_sound_used_by_line(item)
            if sound_list is not None and len(sound_list) > 0:
                return ', '.join([str(i) for i, _ in sound_list])
            else:
                return "--"
        elif column == 3:
            if ssglob.gSpeechMap is None:
                return ''
            try:
                comment = ssglob.gSpeechMap.text_comments[item]
                return str(comment)
            except KeyError:
                return ''

    def ActivateItem(self, event):
        self.parent.parent.buttonTextDescription(None)

    def CreateColumns(self):
        self.InsertColumn(0, "Line")
        self.InsertColumn(1, "Text")
        self.InsertColumn(2, "SFX")
        self.InsertColumn(3, "Description")
        parent = self.GetParent()
        self.SetColumnWidth(0, parent.GetCharWidth() * 7)
        self.SetColumnWidth(1, parent.GetCharWidth() * 42)
        self.SetColumnWidth(2, parent.GetCharWidth() * 7)
        self.SetColumnWidth(3, parent.GetCharWidth() * 42)

    def RegisterEvents(self):
        wx.EVT_LIST_ITEM_SELECTED(self, -1, self.SelectItem)
        wx.EVT_LIST_ITEM_ACTIVATED(self, -1, self.ActivateItem)

    def InitIndexMap(self):
        if ssglob.gTextData is None:
            self.indexMap = 0
        else:
            self.indexMap = range(len(ssglob.gTextData))

    def SortItems(self,sorter=cmp):
        if ssglob.gTextData is None:
            return
        col = self._col
        reverse = not self._colSortFlag[col]
        items = range(len(ssglob.gTextData))
        if col == 0:
            if reverse:
                items.reverse()
        elif col == 1:
            items.sort(key=lambda i: ssglob.gTextData[i], reverse=reverse)
        elif col == 2:
            def sort_by_sounds(i):
                sounds = ssglob.gSpeechMap.get_sound_used_by_line(i)
                if sounds == None or len(sounds) == 0: return -1
                return sounds[0]
            items.sort(key=sort_by_sounds, reverse=reverse)
#        elif col == 3:
#            def sort_by_text_comment(i):
#                try:
#                    return ssglob.gSpeechMap.text_comments[ssglob.gSpeechMap.sounds[i].id]
#                except KeyError:
#                    return -1
#            items.sort(key=sort_by_text_comment, reverse=reverse)
        self.indexMap = items

        # redraw the list
        self.Refresh()


class RightList(BaseList):

    def GetItemCount(self):
        if ssglob.gSpeechMap is None:
            return 0
        return len(ssglob.gSpeechMap)

    def OnGetItemText(self, item, column):
        if ssglob.gSpeechMap is None:
            return None
        item = self.indexMap[item]
        if column == 0:
            return str(item).zfill(4)
        elif column == 1: # ID (actually the original offset, as used by SCUMM)
            id = ssglob.gSpeechMap.sounds[item].id
            return id
        elif column == 2:
            return ', '.join(map(str, ssglob.gSpeechMap.sounds[item].lines_used))
        elif column == 3:
            try:
                comment = ssglob.gSpeechMap.sound_comments[
                    ssglob.gSpeechMap.sounds[item].id
                    ]
                return str(comment)
            except KeyError:
                lu = ssglob.gSpeechMap.sounds[item].lines_used
                if len(lu) > 0:
                    return ssglob.gTextData[lu[0]]
                else:
                    return ''
        elif column == 4:
            return str(ssglob.gSpeechMap.sounds[item].metadata)

    def ActivateItem(self, event):
        item = event.GetIndex()
        ed = wx.TextEntryDialog(self.GetParent(),
                                "Enter a description for the selected sound file.",
                                caption="Enter description",
                                defaultValue=ssglob.gMonsterSou.GetDescription(item))
        if ed.ShowModal() == wx.ID_OK:
            ssglob.gMonsterSou.SetDescription(item, ed.GetValue())
            self.RefreshItem(item)

    def CreateColumns(self):
        self.InsertColumn(0, "Num")
        self.InsertColumn(1, "ID/Original Offset")
        self.InsertColumn(2, "Lines Used")
        self.InsertColumn(3, "Description")
        self.InsertColumn(4, "Metadata")
        parent = self.GetParent()
        self.SetColumnWidth(0, parent.GetCharWidth() * 7)
        self.SetColumnWidth(1, parent.GetCharWidth() * 15)
        self.SetColumnWidth(2, parent.GetCharWidth() * 15)
        self.SetColumnWidth(3, parent.GetCharWidth() * 42)
        self.SetColumnWidth(4, parent.GetCharWidth() * 42)

    def RegisterEvents(self):
        wx.EVT_LIST_ITEM_SELECTED(self, -1, self.SelectItem)
        wx.EVT_LIST_ITEM_ACTIVATED(self, -1, self.ActivateItem)

    def InitIndexMap(self):
        if ssglob.gSpeechMap is None:
            self.indexMap = 0
        else:
            self.indexMap = range(len(ssglob.gSpeechMap.sounds))

    def SortItems(self,sorter=cmp):
        if ssglob.gSpeechMap is None:
            return
        col = self._col
        reverse = not self._colSortFlag[col]
        items = range(len(ssglob.gSpeechMap.sounds))
        if col == 0:
            if reverse:
                items.reverse()
        elif col == 1:
            items.sort(key=lambda i: ssglob.gSpeechMap.sounds[i].id, reverse=reverse)
        elif col == 2:
            items.sort(key=lambda i: ssglob.gSpeechMap.sounds[i].lines_used, reverse=reverse)
        elif col == 3:
            def sort_by_sound_comment(i):
                try:
                    return ssglob.gSpeechMap.sound_comments[ssglob.gSpeechMap.sounds[i].id]
                except KeyError:
                    return -1
            items.sort(key=sort_by_sound_comment, reverse=reverse)
        self.indexMap = items

        # redraw the list
        self.Refresh()